# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...core.request_options import RequestOptions
from ...types.model_version_read import ModelVersionRead

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ModelVersionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ModelVersionRead:
        """
        Deprecated. Use the `deployments/provider-payload` endpoint to fetch information that we send to Model providers.

        Parameters:
            - id: str. A UUID string identifying this model version.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from vellum.client import Vellum

        client = Vellum(
            api_key="YOUR_API_KEY",
        )
        client.model_versions.retrieve(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/", f"v1/model-versions/{jsonable_encoder(id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ModelVersionRead, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncModelVersionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def retrieve(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ModelVersionRead:
        """
        Deprecated. Use the `deployments/provider-payload` endpoint to fetch information that we send to Model providers.

        Parameters:
            - id: str. A UUID string identifying this model version.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from vellum.client import AsyncVellum

        client = AsyncVellum(
            api_key="YOUR_API_KEY",
        )
        await client.model_versions.retrieve(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/", f"v1/model-versions/{jsonable_encoder(id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ModelVersionRead, _response.json())  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ModelVersionExecConfigParameters(pydantic.BaseModel):
    temperature: typing.Optional[float] = None
    max_tokens: typing.Optional[int] = None
    top_p: float
    frequency_penalty: float
    presence_penalty: float
    logit_bias: typing.Optional[typing.Dict[str, typing.Optional[float]]] = None
    stop: typing.Optional[typing.List[str]] = None
    top_k: typing.Optional[float] = None
    custom_parameters: typing.Optional[typing.Dict[str, typing.Any]] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class WorkflowNodeResultEventState(str, enum.Enum):
    """
    - `INITIATED` - INITIATED
    - `STREAMING` - STREAMING
    - `FULFILLED` - FULFILLED
    - `REJECTED` - REJECTED
    """

    INITIATED = "INITIATED"
    STREAMING = "STREAMING"
    FULFILLED = "FULFILLED"
    REJECTED = "REJECTED"

    def visit(
        self,
        initiated: typing.Callable[[], T_Result],
        streaming: typing.Callable[[], T_Result],
        fulfilled: typing.Callable[[], T_Result],
        rejected: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is WorkflowNodeResultEventState.INITIATED:
            return initiated()
        if self is WorkflowNodeResultEventState.STREAMING:
            return streaming()
        if self is WorkflowNodeResultEventState.FULFILLED:
            return fulfilled()
        if self is WorkflowNodeResultEventState.REJECTED:
            return rejected()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .entity_status import EntityStatus
from .environment_enum import EnvironmentEnum

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class DocumentIndexRead(pydantic.BaseModel):
    id: str
    created: dt.datetime
    label: str = pydantic.Field()
    """
    A human-readable label for the document index
    """

    name: str = pydantic.Field()
    """
    A name that uniquely identifies this index within its workspace
    """

    status: typing.Optional[EntityStatus] = pydantic.Field(default=None)
    """
    The current status of the document index
    
    - `ACTIVE` - Active
    - `ARCHIVED` - Archived
    """

    environment: typing.Optional[EnvironmentEnum] = pydantic.Field(default=None)
    """
    The environment this document index is used in
    
    - `DEVELOPMENT` - Development
    - `STAGING` - Staging
    - `PRODUCTION` - Production
    """

    indexing_config: typing.Dict[str, typing.Any] = pydantic.Field()
    """
    Configuration representing how documents should be indexed
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .model_version_build_config import ModelVersionBuildConfig
from .model_version_exec_config import ModelVersionExecConfig
from .model_version_read_status_enum import ModelVersionReadStatusEnum
from .provider_enum import ProviderEnum

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ModelVersionRead(pydantic.BaseModel):
    id: str = pydantic.Field()
    """
    Vellum-generated ID that uniquely identifies this model version.
    """

    created: dt.datetime = pydantic.Field()
    """
    Timestamp of when this model version was created.
    """

    label: str = pydantic.Field()
    """
    Human-friendly name for this model version.
    """

    provider: ProviderEnum = pydantic.Field()
    """
    Which LLM provider this model version is associated with.
    
    - `ANTHROPIC` - Anthropic
    - `AWS_BEDROCK` - AWS Bedrock
    - `AZURE_OPENAI` - Azure OpenAI
    - `COHERE` - Cohere
    - `GOOGLE` - Google
    - `HOSTED` - Hosted
    - `MOSAICML` - MosaicML
    - `OPENAI` - OpenAI
    - `FIREWORKS_AI` - Fireworks AI
    - `HUGGINGFACE` - HuggingFace
    - `MYSTIC` - Mystic
    - `PYQ` - Pyq
    - `REPLICATE` - Replicate
    """

    external_id: str = pydantic.Field()
    """
    The unique id of this model version as it exists in the above provider's system.
    """

    build_config: ModelVersionBuildConfig = pydantic.Field()
    """
    Configuration used to build this model version.
    """

    exec_config: ModelVersionExecConfig = pydantic.Field()
    """
    Configuration used to execute this model version.
    """

    status: typing.Optional[ModelVersionReadStatusEnum] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .search_result import SearchResult

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class SearchNodeResultData(pydantic.BaseModel):
    results_output_id: str
    results: typing.List[SearchResult] = pydantic.Field()
    """
    The results of the search. Each result represents a chunk that matches the search query.
    """

    text_output_id: str
    text: typing.Optional[str] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class WorkflowExecutionEventType(str, enum.Enum):
    """
    - `NODE` - NODE
    - `WORKFLOW` - WORKFLOW
    """

    NODE = "NODE"
    WORKFLOW = "WORKFLOW"

    def visit(self, node: typing.Callable[[], T_Result], workflow: typing.Callable[[], T_Result]) -> T_Result:
        if self is WorkflowExecutionEventType.NODE:
            return node()
        if self is WorkflowExecutionEventType.WORKFLOW:
            return workflow()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class WorkflowExecutionEventErrorCode(str, enum.Enum):
    """
    - `WORKFLOW_INITIALIZATION` - WORKFLOW_INITIALIZATION
    - `WORKFLOW_CANCELLED` - WORKFLOW_CANCELLED
    - `NODE_EXECUTION_COUNT_LIMIT_REACHED` - NODE_EXECUTION_COUNT_LIMIT_REACHED
    - `INTERNAL_SERVER_ERROR` - INTERNAL_SERVER_ERROR
    - `NODE_EXECUTION` - NODE_EXECUTION
    - `LLM_PROVIDER` - LLM_PROVIDER
    - `INVALID_TEMPLATE` - INVALID_TEMPLATE
    - `USER_DEFINED_ERROR` - USER_DEFINED_ERROR
    """

    WORKFLOW_INITIALIZATION = "WORKFLOW_INITIALIZATION"
    WORKFLOW_CANCELLED = "WORKFLOW_CANCELLED"
    NODE_EXECUTION_COUNT_LIMIT_REACHED = "NODE_EXECUTION_COUNT_LIMIT_REACHED"
    INTERNAL_SERVER_ERROR = "INTERNAL_SERVER_ERROR"
    NODE_EXECUTION = "NODE_EXECUTION"
    LLM_PROVIDER = "LLM_PROVIDER"
    INVALID_TEMPLATE = "INVALID_TEMPLATE"
    USER_DEFINED_ERROR = "USER_DEFINED_ERROR"

    def visit(
        self,
        workflow_initialization: typing.Callable[[], T_Result],
        workflow_cancelled: typing.Callable[[], T_Result],
        node_execution_count_limit_reached: typing.Callable[[], T_Result],
        internal_server_error: typing.Callable[[], T_Result],
        node_execution: typing.Callable[[], T_Result],
        llm_provider: typing.Callable[[], T_Result],
        invalid_template: typing.Callable[[], T_Result],
        user_defined_error: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is WorkflowExecutionEventErrorCode.WORKFLOW_INITIALIZATION:
            return workflow_initialization()
        if self is WorkflowExecutionEventErrorCode.WORKFLOW_CANCELLED:
            return workflow_cancelled()
        if self is WorkflowExecutionEventErrorCode.NODE_EXECUTION_COUNT_LIMIT_REACHED:
            return node_execution_count_limit_reached()
        if self is WorkflowExecutionEventErrorCode.INTERNAL_SERVER_ERROR:
            return internal_server_error()
        if self is WorkflowExecutionEventErrorCode.NODE_EXECUTION:
            return node_execution()
        if self is WorkflowExecutionEventErrorCode.LLM_PROVIDER:
            return llm_provider()
        if self is WorkflowExecutionEventErrorCode.INVALID_TEMPLATE:
            return invalid_template()
        if self is WorkflowExecutionEventErrorCode.USER_DEFINED_ERROR:
            return user_defined_error()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class VellumVariableType(str, enum.Enum):
    """
    - `STRING` - STRING
    - `NUMBER` - NUMBER
    - `JSON` - JSON
    - `CHAT_HISTORY` - CHAT_HISTORY
    - `SEARCH_RESULTS` - SEARCH_RESULTS
    - `ERROR` - ERROR
    - `ARRAY` - ARRAY
    - `FUNCTION_CALL` - FUNCTION_CALL
    - `IMAGE` - IMAGE
    """

    STRING = "STRING"
    NUMBER = "NUMBER"
    JSON = "JSON"
    CHAT_HISTORY = "CHAT_HISTORY"
    SEARCH_RESULTS = "SEARCH_RESULTS"
    ERROR = "ERROR"
    ARRAY = "ARRAY"
    FUNCTION_CALL = "FUNCTION_CALL"
    IMAGE = "IMAGE"

    def visit(
        self,
        string: typing.Callable[[], T_Result],
        number: typing.Callable[[], T_Result],
        json: typing.Callable[[], T_Result],
        chat_history: typing.Callable[[], T_Result],
        search_results: typing.Callable[[], T_Result],
        error: typing.Callable[[], T_Result],
        array: typing.Callable[[], T_Result],
        function_call: typing.Callable[[], T_Result],
        image: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is VellumVariableType.STRING:
            return string()
        if self is VellumVariableType.NUMBER:
            return number()
        if self is VellumVariableType.JSON:
            return json()
        if self is VellumVariableType.CHAT_HISTORY:
            return chat_history()
        if self is VellumVariableType.SEARCH_RESULTS:
            return search_results()
        if self is VellumVariableType.ERROR:
            return error()
        if self is VellumVariableType.ARRAY:
            return array()
        if self is VellumVariableType.FUNCTION_CALL:
            return function_call()
        if self is VellumVariableType.IMAGE:
            return image()



# This file was auto-generated by Fern from our API Definition.

from ..core.api_error import ApiError
from ..types.register_prompt_error_response import RegisterPromptErrorResponse


class ConflictError(ApiError):
    def __init__(self, body: RegisterPromptErrorResponse):
        super().__init__(status_code=409, body=body)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class EnvironmentEnum(str, enum.Enum):
    """
    - `DEVELOPMENT` - Development
    - `STAGING` - Staging
    - `PRODUCTION` - Production
    """

    DEVELOPMENT = "DEVELOPMENT"
    STAGING = "STAGING"
    PRODUCTION = "PRODUCTION"

    def visit(
        self,
        development: typing.Callable[[], T_Result],
        staging: typing.Callable[[], T_Result],
        production: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is EnvironmentEnum.DEVELOPMENT:
            return development()
        if self is EnvironmentEnum.STAGING:
            return staging()
        if self is EnvironmentEnum.PRODUCTION:
            return production()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ProcessingFailureReasonEnum(str, enum.Enum):
    """
    - `EXCEEDED_CHARACTER_LIMIT` - Exceeded Character Limit
    - `INVALID_FILE` - Invalid File
    """

    EXCEEDED_CHARACTER_LIMIT = "EXCEEDED_CHARACTER_LIMIT"
    INVALID_FILE = "INVALID_FILE"

    def visit(
        self, exceeded_character_limit: typing.Callable[[], T_Result], invalid_file: typing.Callable[[], T_Result]
    ) -> T_Result:
        if self is ProcessingFailureReasonEnum.EXCEEDED_CHARACTER_LIMIT:
            return exceeded_character_limit()
        if self is ProcessingFailureReasonEnum.INVALID_FILE:
            return invalid_file()



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.conflict_error import ConflictError
from ...errors.not_found_error import NotFoundError
from ...types.provider_enum import ProviderEnum
from ...types.register_prompt_error_response import RegisterPromptErrorResponse
from ...types.register_prompt_model_parameters_request import RegisterPromptModelParametersRequest
from ...types.register_prompt_prompt_info_request import RegisterPromptPromptInfoRequest
from ...types.register_prompt_response import RegisterPromptResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RegisteredPromptsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def register_prompt(
        self,
        *,
        label: str,
        name: str,
        prompt: RegisterPromptPromptInfoRequest,
        provider: typing.Optional[ProviderEnum] = OMIT,
        model: str,
        parameters: RegisterPromptModelParametersRequest,
        meta: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RegisterPromptResponse:
        """
        Registers a prompt within Vellum and creates associated Vellum entities. Intended to be used by integration
        partners, not directly by Vellum users.

        Under the hood, this endpoint creates a new sandbox, a new model version, and a new deployment.

        Parameters:
            - label: str. A human-friendly label for corresponding entities created in Vellum.

            - name: str. A uniquely-identifying name for corresponding entities created in Vellum.

            - prompt: RegisterPromptPromptInfoRequest. Information about how to execute the prompt template.

            - provider: typing.Optional[ProviderEnum]. The initial LLM provider to use for this prompt

                                                       * `ANTHROPIC` - Anthropic
                                                       * `AWS_BEDROCK` - AWS Bedrock
                                                       * `AZURE_OPENAI` - Azure OpenAI
                                                       * `COHERE` - Cohere
                                                       * `GOOGLE` - Google
                                                       * `HOSTED` - Hosted
                                                       * `MOSAICML` - MosaicML
                                                       * `OPENAI` - OpenAI
                                                       * `FIREWORKS_AI` - Fireworks AI
                                                       * `HUGGINGFACE` - HuggingFace
                                                       * `MYSTIC` - Mystic
                                                       * `PYQ` - Pyq
                                                       * `REPLICATE` - Replicate
            - model: str. The initial model to use for this prompt

            - parameters: RegisterPromptModelParametersRequest. The initial model parameters to use for  this prompt

            - meta: typing.Optional[typing.Dict[str, typing.Any]]. Optionally include additional metadata to store along with the prompt.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from vellum import (
            BlockTypeEnum,
            PromptTemplateBlockDataRequest,
            PromptTemplateBlockPropertiesRequest,
            PromptTemplateBlockRequest,
            RegisteredPromptInputVariableRequest,
            RegisterPromptModelParametersRequest,
            RegisterPromptPromptInfoRequest,
        )
        from vellum.client import Vellum

        client = Vellum(
            api_key="YOUR_API_KEY",
        )
        client.registered_prompts.register_prompt(
            label="label",
            name="name",
            prompt=RegisterPromptPromptInfoRequest(
                prompt_block_data=PromptTemplateBlockDataRequest(
                    version=1,
                    blocks=[
                        PromptTemplateBlockRequest(
                            id="id",
                            block_type=BlockTypeEnum.CHAT_MESSAGE,
                            properties=PromptTemplateBlockPropertiesRequest(),
                        )
                    ],
                ),
                input_variables=[
                    RegisteredPromptInputVariableRequest(
                        key="key",
                    )
                ],
            ),
            model="model",
            parameters=RegisterPromptModelParametersRequest(
                temperature=1.1,
                max_tokens=1,
                top_p=1.1,
                frequency_penalty=1.1,
                presence_penalty=1.1,
            ),
        )
        """
        _request: typing.Dict[str, typing.Any] = {
            "label": label,
            "name": name,
            "prompt": prompt,
            "model": model,
            "parameters": parameters,
        }
        if provider is not OMIT:
            _request["provider"] = provider
        if meta is not OMIT:
            _request["meta"] = meta
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/", "v1/registered-prompts/register"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RegisterPromptResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(RegisterPromptErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRegisteredPromptsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def register_prompt(
        self,
        *,
        label: str,
        name: str,
        prompt: RegisterPromptPromptInfoRequest,
        provider: typing.Optional[ProviderEnum] = OMIT,
        model: str,
        parameters: RegisterPromptModelParametersRequest,
        meta: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RegisterPromptResponse:
        """
        Registers a prompt within Vellum and creates associated Vellum entities. Intended to be used by integration
        partners, not directly by Vellum users.

        Under the hood, this endpoint creates a new sandbox, a new model version, and a new deployment.

        Parameters:
            - label: str. A human-friendly label for corresponding entities created in Vellum.

            - name: str. A uniquely-identifying name for corresponding entities created in Vellum.

            - prompt: RegisterPromptPromptInfoRequest. Information about how to execute the prompt template.

            - provider: typing.Optional[ProviderEnum]. The initial LLM provider to use for this prompt

                                                       * `ANTHROPIC` - Anthropic
                                                       * `AWS_BEDROCK` - AWS Bedrock
                                                       * `AZURE_OPENAI` - Azure OpenAI
                                                       * `COHERE` - Cohere
                                                       * `GOOGLE` - Google
                                                       * `HOSTED` - Hosted
                                                       * `MOSAICML` - MosaicML
                                                       * `OPENAI` - OpenAI
                                                       * `FIREWORKS_AI` - Fireworks AI
                                                       * `HUGGINGFACE` - HuggingFace
                                                       * `MYSTIC` - Mystic
                                                       * `PYQ` - Pyq
                                                       * `REPLICATE` - Replicate
            - model: str. The initial model to use for this prompt

            - parameters: RegisterPromptModelParametersRequest. The initial model parameters to use for  this prompt

            - meta: typing.Optional[typing.Dict[str, typing.Any]]. Optionally include additional metadata to store along with the prompt.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from vellum import (
            BlockTypeEnum,
            PromptTemplateBlockDataRequest,
            PromptTemplateBlockPropertiesRequest,
            PromptTemplateBlockRequest,
            RegisteredPromptInputVariableRequest,
            RegisterPromptModelParametersRequest,
            RegisterPromptPromptInfoRequest,
        )
        from vellum.client import AsyncVellum

        client = AsyncVellum(
            api_key="YOUR_API_KEY",
        )
        await client.registered_prompts.register_prompt(
            label="label",
            name="name",
            prompt=RegisterPromptPromptInfoRequest(
                prompt_block_data=PromptTemplateBlockDataRequest(
                    version=1,
                    blocks=[
                        PromptTemplateBlockRequest(
                            id="id",
                            block_type=BlockTypeEnum.CHAT_MESSAGE,
                            properties=PromptTemplateBlockPropertiesRequest(),
                        )
                    ],
                ),
                input_variables=[
                    RegisteredPromptInputVariableRequest(
                        key="key",
                    )
                ],
            ),
            model="model",
            parameters=RegisterPromptModelParametersRequest(
                temperature=1.1,
                max_tokens=1,
                top_p=1.1,
                frequency_penalty=1.1,
                presence_penalty=1.1,
            ),
        )
        """
        _request: typing.Dict[str, typing.Any] = {
            "label": label,
            "name": name,
            "prompt": prompt,
            "model": model,
            "parameters": parameters,
        }
        if provider is not OMIT:
            _request["provider"] = provider
        if meta is not OMIT:
            _request["meta"] = meta
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_environment().default}/", "v1/registered-prompts/register"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(RegisterPromptResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(typing.Any, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(RegisterPromptErrorResponse, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .chat_message_role import ChatMessageRole
from .vellum_variable_type import VellumVariableType

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class PromptTemplateBlockProperties(pydantic.BaseModel):
    chat_role: typing.Optional[ChatMessageRole] = None
    chat_message_unterminated: typing.Optional[bool] = None
    chat_source: typing.Optional[str] = None
    template: typing.Optional[str] = None
    template_type: typing.Optional[VellumVariableType] = None
    function_name: typing.Optional[str] = None
    function_description: typing.Optional[str] = None
    function_parameters: typing.Optional[typing.Dict[str, typing.Any]] = None
    function_forced: typing.Optional[bool] = None
    blocks: typing.Optional[typing.List[PromptTemplateBlock]] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}


from .prompt_template_block import PromptTemplateBlock  # noqa: E402

PromptTemplateBlockProperties.update_forward_refs()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .model_version_exec_config_parameters import ModelVersionExecConfigParameters
from .prompt_template_block_data import PromptTemplateBlockData
from .vellum_variable import VellumVariable

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ModelVersionExecConfig(pydantic.BaseModel):
    parameters: ModelVersionExecConfigParameters = pydantic.Field()
    """
    The generation parameters that are passed to the LLM provider at runtime.
    """

    input_variables: typing.List[VellumVariable] = pydantic.Field()
    """
    Input variables specified in the prompt template.
    """

    prompt_template: typing.Optional[str] = pydantic.Field(default=None)
    """
    The template used to generate prompts for this model version.
    """

    prompt_block_data: typing.Optional[PromptTemplateBlockData] = None
    prompt_syntax_version: typing.Optional[int] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class TestSuiteRunState(str, enum.Enum):
    """
    - `QUEUED` - Queued
    - `RUNNING` - Running
    - `COMPLETE` - Complete
    - `FAILED` - Failed
    - `CANCELLED` - Cancelled
    """

    QUEUED = "QUEUED"
    RUNNING = "RUNNING"
    COMPLETE = "COMPLETE"
    FAILED = "FAILED"
    CANCELLED = "CANCELLED"

    def visit(
        self,
        queued: typing.Callable[[], T_Result],
        running: typing.Callable[[], T_Result],
        complete: typing.Callable[[], T_Result],
        failed: typing.Callable[[], T_Result],
        cancelled: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is TestSuiteRunState.QUEUED:
            return queued()
        if self is TestSuiteRunState.RUNNING:
            return running()
        if self is TestSuiteRunState.COMPLETE:
            return complete()
        if self is TestSuiteRunState.FAILED:
            return failed()
        if self is TestSuiteRunState.CANCELLED:
            return cancelled()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ProviderEnum(str, enum.Enum):
    """
    - `ANTHROPIC` - Anthropic
    - `AWS_BEDROCK` - AWS Bedrock
    - `AZURE_OPENAI` - Azure OpenAI
    - `COHERE` - Cohere
    - `GOOGLE` - Google
    - `HOSTED` - Hosted
    - `MOSAICML` - MosaicML
    - `OPENAI` - OpenAI
    - `FIREWORKS_AI` - Fireworks AI
    - `HUGGINGFACE` - HuggingFace
    - `MYSTIC` - Mystic
    - `PYQ` - Pyq
    - `REPLICATE` - Replicate
    """

    ANTHROPIC = "ANTHROPIC"
    AWS_BEDROCK = "AWS_BEDROCK"
    AZURE_OPENAI = "AZURE_OPENAI"
    COHERE = "COHERE"
    GOOGLE = "GOOGLE"
    HOSTED = "HOSTED"
    MOSAICML = "MOSAICML"
    OPENAI = "OPENAI"
    FIREWORKS_AI = "FIREWORKS_AI"
    HUGGINGFACE = "HUGGINGFACE"
    MYSTIC = "MYSTIC"
    PYQ = "PYQ"
    REPLICATE = "REPLICATE"

    def visit(
        self,
        anthropic: typing.Callable[[], T_Result],
        aws_bedrock: typing.Callable[[], T_Result],
        azure_openai: typing.Callable[[], T_Result],
        cohere: typing.Callable[[], T_Result],
        google: typing.Callable[[], T_Result],
        hosted: typing.Callable[[], T_Result],
        mosaicml: typing.Callable[[], T_Result],
        openai: typing.Callable[[], T_Result],
        fireworks_ai: typing.Callable[[], T_Result],
        huggingface: typing.Callable[[], T_Result],
        mystic: typing.Callable[[], T_Result],
        pyq: typing.Callable[[], T_Result],
        replicate: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ProviderEnum.ANTHROPIC:
            return anthropic()
        if self is ProviderEnum.AWS_BEDROCK:
            return aws_bedrock()
        if self is ProviderEnum.AZURE_OPENAI:
            return azure_openai()
        if self is ProviderEnum.COHERE:
            return cohere()
        if self is ProviderEnum.GOOGLE:
            return google()
        if self is ProviderEnum.HOSTED:
            return hosted()
        if self is ProviderEnum.MOSAICML:
            return mosaicml()
        if self is ProviderEnum.OPENAI:
            return openai()
        if self is ProviderEnum.FIREWORKS_AI:
            return fireworks_ai()
        if self is ProviderEnum.HUGGINGFACE:
            return huggingface()
        if self is ProviderEnum.MYSTIC:
            return mystic()
        if self is ProviderEnum.PYQ:
            return pyq()
        if self is ProviderEnum.REPLICATE:
            return replicate()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .chat_message_request import ChatMessageRequest

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class GenerateRequest(pydantic.BaseModel):
    input_values: typing.Dict[str, typing.Any] = pydantic.Field()
    """
    Key/value pairs for each template variable defined in the deployment's prompt.
    """

    chat_history: typing.Optional[typing.List[ChatMessageRequest]] = pydantic.Field(default=None)
    """
    Optionally provide a list of chat messages that'll be used in place of the special chat_history variable, if included in the prompt.
    """

    external_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Optionally include a unique identifier for each generation, as represented outside of Vellum. Note that this should generally be a list of length one.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class BlockTypeEnum(str, enum.Enum):
    """
    - `CHAT_MESSAGE` - CHAT_MESSAGE
    - `CHAT_HISTORY` - CHAT_HISTORY
    - `JINJA` - JINJA
    - `FUNCTION_DEFINITION` - FUNCTION_DEFINITION
    """

    CHAT_MESSAGE = "CHAT_MESSAGE"
    CHAT_HISTORY = "CHAT_HISTORY"
    JINJA = "JINJA"
    FUNCTION_DEFINITION = "FUNCTION_DEFINITION"

    def visit(
        self,
        chat_message: typing.Callable[[], T_Result],
        chat_history: typing.Callable[[], T_Result],
        jinja: typing.Callable[[], T_Result],
        function_definition: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is BlockTypeEnum.CHAT_MESSAGE:
            return chat_message()
        if self is BlockTypeEnum.CHAT_HISTORY:
            return chat_history()
        if self is BlockTypeEnum.JINJA:
            return jinja()
        if self is BlockTypeEnum.FUNCTION_DEFINITION:
            return function_definition()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class MetadataFilterRuleCombinator(str, enum.Enum):
    """
    - `and` - AND
    - `or` - OR
    """

    AND = "and"
    OR = "or"

    def visit(self, and_: typing.Callable[[], T_Result], or_: typing.Callable[[], T_Result]) -> T_Result:
        if self is MetadataFilterRuleCombinator.AND:
            return and_()
        if self is MetadataFilterRuleCombinator.OR:
            return or_()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class VellumErrorCodeEnum(str, enum.Enum):
    """
    - `INVALID_REQUEST` - INVALID_REQUEST
    - `PROVIDER_ERROR` - PROVIDER_ERROR
    - `INTERNAL_SERVER_ERROR` - INTERNAL_SERVER_ERROR
    - `USER_DEFINED_ERROR` - USER_DEFINED_ERROR
    """

    INVALID_REQUEST = "INVALID_REQUEST"
    PROVIDER_ERROR = "PROVIDER_ERROR"
    INTERNAL_SERVER_ERROR = "INTERNAL_SERVER_ERROR"
    USER_DEFINED_ERROR = "USER_DEFINED_ERROR"

    def visit(
        self,
        invalid_request: typing.Callable[[], T_Result],
        provider_error: typing.Callable[[], T_Result],
        internal_server_error: typing.Callable[[], T_Result],
        user_defined_error: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is VellumErrorCodeEnum.INVALID_REQUEST:
            return invalid_request()
        if self is VellumErrorCodeEnum.PROVIDER_ERROR:
            return provider_error()
        if self is VellumErrorCodeEnum.INTERNAL_SERVER_ERROR:
            return internal_server_error()
        if self is VellumErrorCodeEnum.USER_DEFINED_ERROR:
            return user_defined_error()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class LogicalOperator(str, enum.Enum):
    """
    - `=` - EQUALS
    - `!=` - DOES_NOT_EQUAL
    - `<` - LESS_THAN
    - `>` - GREATER_THAN
    - `<=` - LESS_THAN_OR_EQUAL_TO
    - `>=` - GREATER_THAN_OR_EQUAL_TO
    - `contains` - CONTAINS
    - `beginsWith` - BEGINS_WITH
    - `endsWith` - ENDS_WITH
    - `doesNotContain` - DOES_NOT_CONTAIN
    - `doesNotBeginWith` - DOES_NOT_BEGIN_WITH
    - `doesNotEndWith` - DOES_NOT_END_WITH
    - `null` - NULL
    - `notNull` - NOT_NULL
    - `in` - IN
    - `notIn` - NOT_IN
    - `between` - BETWEEN
    - `notBetween` - NOT_BETWEEN
    """

    EQUALS = "="
    """
    Equals
    """

    DOES_NOT_EQUAL = "!="
    """
    Does not equal
    """

    LESS_THAN = "<"
    """
    Less than
    """

    GREATER_THAN = ">"
    """
    Greater than
    """

    LESS_THAN_OR_EQUAL_TO = "<="
    """
    Less than or equal to
    """

    GREATER_THAN_OR_EQUAL_TO = ">="
    """
    Greater than or equal to
    """

    CONTAINS = "contains"
    """
    Contains
    """

    BEGINS_WITH = "beginsWith"
    """
    Begins with
    """

    ENDS_WITH = "endsWith"
    """
    Ends with
    """

    DOES_NOT_CONTAIN = "doesNotContain"
    """
    Does not contain
    """

    DOES_NOT_BEGIN_WITH = "doesNotBeginWith"
    """
    Does not begin with
    """

    DOES_NOT_END_WITH = "doesNotEndWith"
    """
    Does not end with
    """

    NULL = "null"
    """
    Null
    """

    NOT_NULL = "notNull"
    """
    Not null
    """

    IN = "in"
    """
    In
    """

    NOT_IN = "notIn"
    """
    Not in
    """

    BETWEEN = "between"
    """
    Between
    """

    NOT_BETWEEN = "notBetween"
    """
    Not between
    """

    def visit(
        self,
        equals: typing.Callable[[], T_Result],
        does_not_equal: typing.Callable[[], T_Result],
        less_than: typing.Callable[[], T_Result],
        greater_than: typing.Callable[[], T_Result],
        less_than_or_equal_to: typing.Callable[[], T_Result],
        greater_than_or_equal_to: typing.Callable[[], T_Result],
        contains: typing.Callable[[], T_Result],
        begins_with: typing.Callable[[], T_Result],
        ends_with: typing.Callable[[], T_Result],
        does_not_contain: typing.Callable[[], T_Result],
        does_not_begin_with: typing.Callable[[], T_Result],
        does_not_end_with: typing.Callable[[], T_Result],
        null: typing.Callable[[], T_Result],
        not_null: typing.Callable[[], T_Result],
        in_: typing.Callable[[], T_Result],
        not_in: typing.Callable[[], T_Result],
        between: typing.Callable[[], T_Result],
        not_between: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is LogicalOperator.EQUALS:
            return equals()
        if self is LogicalOperator.DOES_NOT_EQUAL:
            return does_not_equal()
        if self is LogicalOperator.LESS_THAN:
            return less_than()
        if self is LogicalOperator.GREATER_THAN:
            return greater_than()
        if self is LogicalOperator.LESS_THAN_OR_EQUAL_TO:
            return less_than_or_equal_to()
        if self is LogicalOperator.GREATER_THAN_OR_EQUAL_TO:
            return greater_than_or_equal_to()
        if self is LogicalOperator.CONTAINS:
            return contains()
        if self is LogicalOperator.BEGINS_WITH:
            return begins_with()
        if self is LogicalOperator.ENDS_WITH:
            return ends_with()
        if self is LogicalOperator.DOES_NOT_CONTAIN:
            return does_not_contain()
        if self is LogicalOperator.DOES_NOT_BEGIN_WITH:
            return does_not_begin_with()
        if self is LogicalOperator.DOES_NOT_END_WITH:
            return does_not_end_with()
        if self is LogicalOperator.NULL:
            return null()
        if self is LogicalOperator.NOT_NULL:
            return not_null()
        if self is LogicalOperator.IN:
            return in_()
        if self is LogicalOperator.NOT_IN:
            return not_in()
        if self is LogicalOperator.BETWEEN:
            return between()
        if self is LogicalOperator.NOT_BETWEEN:
            return not_between()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .document_document_to_document_index import DocumentDocumentToDocumentIndex
from .document_status import DocumentStatus
from .processing_failure_reason_enum import ProcessingFailureReasonEnum
from .processing_state_enum import ProcessingStateEnum

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class SlimDocument(pydantic.BaseModel):
    id: str = pydantic.Field()
    """
    Vellum-generated ID that uniquely identifies this document.
    """

    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The external ID that was originally provided when uploading the document.
    """

    last_uploaded_at: dt.datetime = pydantic.Field()
    """
    A timestamp representing when this document was most recently uploaded.
    """

    label: str = pydantic.Field()
    """
    Human-friendly name for this document.
    """

    processing_state: typing.Optional[ProcessingStateEnum] = pydantic.Field(default=None)
    """
    An enum value representing where this document is along its processing lifecycle. Note that this is different than its indexing lifecycle.
    
    - `QUEUED` - Queued
    - `PROCESSING` - Processing
    - `PROCESSED` - Processed
    - `FAILED` - Failed
    """

    processing_failure_reason: typing.Optional[ProcessingFailureReasonEnum] = pydantic.Field(default=None)
    """
    An enum value representing why the document could not be processed. Is null unless processing_state is FAILED.
    
    - `EXCEEDED_CHARACTER_LIMIT` - Exceeded Character Limit
    - `INVALID_FILE` - Invalid File
    """

    status: typing.Optional[DocumentStatus] = pydantic.Field(default=None)
    """
    The document's current status.
    
    - `ACTIVE` - Active
    """

    keywords: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of keywords associated with this document. Originally provided when uploading the document.
    """

    metadata: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    A previously supplied JSON object containing metadata that can be filtered on when searching.
    """

    document_to_document_indexes: typing.List[DocumentDocumentToDocumentIndex]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}
